@@ -4,8 +4,112 @@
  * Copyright (C) 1998 the Free Software Foundation
  * Author: Jonathan Blandford <jrb@redhat.com>
  * Based on code by: Federico Mena (Quartic), Szekeres Istvan (Pista)
+
+
+Ghostview interface to ghostscript
+
+When the GHOSTVIEW environment variable is set, ghostscript draws on
+an existing drawable rather than creating its own window.  Ghostscript
+can be directed to draw on either a window or a pixmap.
+
+Drawing on a Window
+
+The GHOSTVIEW environment variable contains the window id of the target
+window.  The window id is an integer.  Ghostscript will use the attributes
+of the window to obtain the width, height, colormap, screen, and visual of
+the window. The remainder of the information is gotten from the GHOSTVIEW
+property on that window.
+
+
+Drawing on a Pixmap
+
+The GHOSTVIEW environment variable contains a window id and a pixmap id.
+They are integers separated by white space.  Ghostscript will use the
+attributes of the window to obtain the colormap, screen, and visual to use.
+The width and height will be obtained from the pixmap. The remainder of the
+information, is gotten from the GHOSTVIEW property on the window.  In this
+case, the property is deleted when read.
+
+The GHOSTVIEW environment variable
+
+parameters:	window-id [pixmap-id]
+
+scanf format:	"%d %d"
+
+explanation of parameters:
+
+	window-id: tells ghostscript where to
+		    - read the GHOSTVIEW property
+		    - send events
+		    If pixmap-id is not present,
+		    ghostscript will draw on this window.
+
+	pixmap-id: If present, tells ghostscript that a pixmap will be used
+		    as the final destination for drawing.  The window will
+		    not be touched for drawing purposes.
+
+The GHOSTVIEW property
+
+type:	STRING
+
+parameters:
+
+    bpixmap orient llx lly urx ury xdpi ydpi [left bottom top right]
+
+scanf format: "%d %d %d %d %d %d %f %f %d %d %d %d"
+
+explanation of parameters:
+
+	bpixmap: pixmap id of the backing pixmap for the window.  If no
+		pixmap is to be used, this parameter should be zero.  This
+		parameter must be zero when drawing on a pixmap.
+
+	orient:	orientation of the page.  The number represents clockwise
+		rotation of the paper in degrees.  Permitted values are
+		0, 90, 180, 270.
+
+	llx, lly, urx, ury: Bounding box of the drawable.  The bounding box
+		is specified in PostScript points in default user coordinates.
+
+	xdpi, ydpi: Resolution of window.  (This can be derived from the
+		other parameters, but not without roundoff error.  These
+		values are included to avoid this error.)
+
+	left, bottom, top, right: (optional)
+		Margins around the window.  The margins extend the imageable
+		area beyond the boundaries of the window.  This is primarily
+		used for popup zoom windows.  I have encountered several
+		instances of PostScript programs that position themselves
+		with respect to the imageable area.  The margins are specified
+		in PostScript points.  If omitted, the margins are assumed to
+		be 0.
+
+Events from ghostscript
+
+If the final destination is a pixmap, the client will get a property notify
+event when ghostscript reads the GHOSTVIEW property causing it to be deleted.
+
+Ghostscript sends events to the window where it read the GHOSTVIEW property.
+These events are of type ClientMessage.  The message_type is set to
+either PAGE or DONE.  The first long data value gives the window to be used
+to send replies to ghostscript.  The second long data value gives the primary
+drawable.  If rendering to a pixmap, it is the primary drawable.  If rendering
+to a window, the backing pixmap is the primary drawable.  If no backing pixmap
+is employed, then the window is the primary drawable.  This field is necessary
+to distinguish multiple ghostscripts rendering to separate pixmaps where the
+GHOSTVIEW property was placed on the same window.
+
+The PAGE message indicates that a "page" has completed.  Ghostscript will
+wait until it receives a ClientMessage whose message_type is NEXT before
+continuing.
+
+The DONE message indicates that ghostscript has finished processing.
+
+
+
  */
 
+
 #include "config.h"
 #include <string.h>
 #include <signal.h>
@@ -23,6 +127,18 @@
 #include "gtkgs.h"
 #include "file.h"
 
+#define NON_BLOCKING_IO
+#ifdef NON_BLOCKING_IO
+#include <fcntl.h>
+/* if POSIX O_NONBLOCK is not available, use O_NDELAY */
+#if !defined(O_NONBLOCK) && defined(O_NDELAY)
+#define O_NONBLOCK O_NDELAY
+#endif
+#endif
+
+
+
+
 #define SIGVAL void
 
 #define RES 75
@@ -174,6 +290,59 @@ gtk_gs_destroy (GtkObject *object)
 }
 
 
+/* FIXME: I'm not sure if all this is supposed to be here 
+ * this is just a quick hack so that this can be called whenever
+ * something changes.
+ */
+static void
+gtk_gs_munge_adjustments (GtkGS 	*gs)
+{
+        gint x, y;
+
+	gdk_window_get_position (gs->pstarget, &x, &y);
+
+	/* g_print("Munge adj: waw=%d gsw=%d hau=%d hal=%d \n", gs->widget.allocation.width, gs->width, gs->hadj->upper, gs->hadj->lower);
+	 */
+
+	/* 
+	 * This is a bit messy:
+	 * we want to make sure that we do the right thing if dragged.
+	 */
+
+	if (gs->widget.allocation.width >= gs->width) {
+		x = (gs->widget.allocation.width - gs->width)/2;
+		gs->hadj->value = (gs->hadj->upper - gs->hadj->lower)/2;
+		gs->hadj->page_size = gs->hadj->upper - gs->hadj->lower;
+	} else {
+		if (x > 0)
+			x = 0;
+		else if (gs->widget.allocation.width > x + gs->width) 
+			x = gs->widget.allocation.width - gs->width;
+		gs->hadj->page_size = ((gfloat) gs->widget.allocation.width)/gs->width;
+		gs->hadj->value = (gs->hadj->upper-gs->hadj->lower) *
+			((gfloat) (gs->widget.allocation.width/2 - x))/gs->width;
+	}
+	if (gs->widget.allocation.height >= gs->height) {
+		y = (gs->widget.allocation.height - gs->height)/2;
+		gs->vadj->value = (gs->vadj->upper - gs->vadj->lower)/2;
+		gs->vadj->page_size = gs->vadj->upper - gs->vadj->lower;
+	} else {
+		if (y > 0)
+			y = 0;
+		else if (gs->widget.allocation.height > y + gs->height) 
+			y = gs->widget.allocation.height - gs->height;
+		gs->vadj->page_size = ((gfloat) gs->widget.allocation.height)/gs->height;
+		gs->vadj->value = (gs->vadj->upper-gs->vadj->lower) *
+			((gfloat) (gs->widget.allocation.height/2 - y))/gs->height;
+	}
+
+	/* now we want to update the slider. */
+	gdk_window_move (gs->pstarget, x, y);
+	gtk_adjustment_changed (gs->hadj);
+	gtk_adjustment_changed (gs->vadj);
+}
+			  
+
 static void
 gtk_gs_realize (GtkWidget *widget)
 {
@@ -219,6 +388,10 @@ gtk_gs_realize (GtkWidget *widget)
 
         gs->width = attributes.width;
         gs->height = attributes.height;
+	/* XXX ??? */
+        if ((gs->width > 0) && (gs->height >0)) {
+		gtk_gs_munge_adjustments (gs);
+        }
         /* XXX: how to handle X BadAlloc and stuff? */
         gtk_signal_connect(GTK_OBJECT(widget), "event",
                            GTK_SIGNAL_FUNC(gtk_gs_widget_event), gs);
@@ -230,7 +403,6 @@ gtk_gs_size_allocate (GtkWidget     *widget,
 {
         GtkGS *gs = GTK_GS (widget);
         GdkEventConfigure event;
-        gint x, y;
         
         g_return_if_fail (widget != NULL);
         g_return_if_fail (GTK_IS_GS (widget));
@@ -256,44 +428,7 @@ gtk_gs_size_allocate (GtkWidget     *widget,
          * update the adjustment if necessary (ie. a resize);
          */
         if ((gs->width > 0) && (gs->height >0)) {
-                gdk_window_get_position (gs->pstarget, &x, &y);
-
-                /* 
-                 * This is a bit messy:
-                 * we want to make sure that we do the right thing if dragged.
-                 */
-
-                if (gs->widget.allocation.width >= gs->width) {
-                        x = (gs->widget.allocation.width - gs->width)/2;
-                        gs->hadj->value = (gs->hadj->upper - gs->hadj->lower)/2;
-                        gs->hadj->page_size = gs->hadj->upper - gs->hadj->lower;
-                } else {
-                        if (x > 0)
-                                x = 0;
-                        else if (gs->widget.allocation.width > x + gs->width) 
-                                x = gs->widget.allocation.width - gs->width;
-                        gs->hadj->page_size = ((gfloat) gs->widget.allocation.width)/gs->width;
-                        gs->hadj->value = (gs->hadj->upper-gs->hadj->lower) *
-                                ((gfloat) (gs->widget.allocation.width/2 - x))/gs->width;
-                }
-                if (gs->widget.allocation.height >= gs->height) {
-                        y = (gs->widget.allocation.height - gs->height)/2;
-                        gs->vadj->value = (gs->vadj->upper - gs->vadj->lower)/2;
-                        gs->vadj->page_size = gs->vadj->upper - gs->vadj->lower;
-                } else {
-                        if (y > 0)
-                                y = 0;
-                        else if (gs->widget.allocation.height > y + gs->height) 
-                                y = gs->widget.allocation.height - gs->height;
-                        gs->vadj->page_size = ((gfloat) gs->widget.allocation.height)/gs->height;
-                        gs->vadj->value = (gs->vadj->upper-gs->vadj->lower) *
-                                ((gfloat) (gs->widget.allocation.height/2 - y))/gs->height;
-                }
-
-                /* now we want to update the slider. */
-                gdk_window_move (gs->pstarget, x, y);
-                gtk_adjustment_changed (gs->hadj);
-                gtk_adjustment_changed (gs->vadj);
+		gtk_gs_munge_adjustments (gs);
         }
 
 }
@@ -304,15 +439,26 @@ gtk_gs_widget_event(GtkWidget *widget,
                     gpointer data)
 {
         GtkGS *gs = (GtkGS *)data;
-        
+
+
+
 	if (event->type != GDK_CLIENT_EVENT) 
                 return FALSE;
-	
+
+        /* the first long is the window to communicate with gs,
+         only if event if client_event */
+
+        gs->message_window = event->client.data.l[0];
+        g_print("Message window %ld\n", gs->message_window);
+
 	if (event->client.message_type == gs_class->page_atom) {
+                g_print("Interpreter completed: busy -> FALSE\n");
                 gs->busy = FALSE;
 	}
         return TRUE;
 }
+
+
 static void
 gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data)
 {
@@ -430,7 +576,10 @@ set_up_page (GtkGS *gs)
         guint orientation;
         char               buf[1024];
         GdkPixmapPrivate  *pprivate;
+        gint w, h;
 
+        /* Do we have to check if the actual geometry changed? */
+        
         stop_interpreter(gs);
         if (gs->doc == NULL) {
                 /* 
@@ -448,20 +597,32 @@ set_up_page (GtkGS *gs)
                 orientation = GTK_GS_ORIENTATION_PORTRAIT;
         else
                 orientation = gs->orientation;
-        /* do we need to make a new pixmap??? */
+
+
         if (!gs->use_bpixmap) {
-                
-        }else if ((gs->use_bpixmap) && compute_size (gs)) {
+                g_print("Do not create pixmap\n");
+        }
+        else if ((gs->use_bpixmap) && compute_size (gs)) { 
+                g_print("Creating pixmap\n");
+
+                gdk_flush(); 
+
                 if (gs->bpixmap) {
                         gdk_pixmap_unref(gs->bpixmap);
                         gs->bpixmap = NULL;
                 }
                 gs->bpixmap = gdk_pixmap_new(gs->pstarget, gs->width, gs->height, -1);
                 gdk_window_set_back_pixmap(gs->pstarget, gs->bpixmap, FALSE);
+
+                gdk_window_resize (gs->pstarget, gs->width, gs->height); 
+
+                /* FIX ME 
+                At this point the old pixmap is still in the background. We should clean 
+                it up before we continue. 
+                */
+
         }
 
-        gdk_window_resize (gs->pstarget, gs->width, gs->height);
-        
         pprivate = (GdkPixmapPrivate *) gs->bpixmap;
         snprintf(buf, 1024, "%ld %d %d %d %d %d %f %f %d %d %d %d",
                  (pprivate ? pprivate->xwindow : 0L),
@@ -474,7 +635,7 @@ set_up_page (GtkGS *gs)
                  gs->bottom_margin,
                  gs->right_margin,
                  gs->top_margin);
-        
+        g_print("Sent to PostScript %s\n",buf);
         gdk_property_change(gs->pstarget,
                             gs_class->gs_atom,
                             gs_class->string_atom,
@@ -482,7 +643,9 @@ set_up_page (GtkGS *gs)
                             GDK_PROP_MODE_REPLACE,
                             buf,
                             strlen(buf));
-        gtk_gs_center_page (gs);
+        gdk_flush(); 
+
+        //        gtk_gs_center_page (gs);
 }
 
 static void
@@ -495,6 +658,8 @@ close_pipe(int p[2])
 static gboolean
 is_interpreter_ready(GtkGS *gs)
 {
+        g_print("Is interpreter ready pid %d busy %d %lx [%s]", 
+                 gs->interpreter_pid,gs->busy, gs->ps_input,gs->ps_input);
         return (gs->interpreter_pid != -1 &&
                 !gs->busy &&
                 gs->ps_input == NULL);
@@ -714,11 +879,15 @@ start_interpreter (GtkGS *gs)
                         dup2(std_in[0], 0);
                         close(std_in[0]);
                 } else if (strcmp(gs->gs_filename, "-")) {
+                        /* What about PDF? */
                         dup2(std_in[0], 0);
                         close(std_in[0]);
                 }
 
                 execvp(argv[0], argv);
+                
+                /* Notify error */
+                g_print("Unable to execute [%s]\n", argv[0]);
 		_exit(1);
 		break;
 	default: /* parent */
@@ -881,7 +1050,7 @@ gtk_gs_new_from_file(GtkAdjustment *hadj, GtkAdjustment *vadj, char *fname)
 	return gs;
 }
 
-void
+gboolean
 gtk_gs_load (GtkGS *gs, char *fname)
 {
         struct stat sbuf;
@@ -890,6 +1059,7 @@ gtk_gs_load (GtkGS *gs, char *fname)
         printf (" in gs_load\n");
 	if (fname == NULL) {
                 /* clear drawing area? */
+                return FALSE;
 	} else {
                 /*
                  * We need to make sure that the file is loadable/exists!
@@ -910,8 +1080,11 @@ gtk_gs_load (GtkGS *gs, char *fname)
 		gs->doc = NULL;
 
 		/* prepare this document */
-		gs->gs_psfile = fopen(fname, "r");
+		if ((gs->gs_psfile = fopen(fname, "r")) == NULL)
+                        return FALSE;
+
                 gs->gs_filename = g_strdup(fname);
+
                 printf (" filename is: %s\n", gs->gs_filename);
 		gs->gs_filename_raw = g_strdup(fname);
                 gs->current_page = -1;
@@ -924,9 +1097,28 @@ gtk_gs_load (GtkGS *gs, char *fname)
                                  "/tmp/",
                                  &gs->gs_filename_dsc, gs->gs_gs_cmd_scan_pdf,
                                  &gs->gs_filename_unc, gs->gs_uncompress_command);
+                if (gs->doc == NULL) {
+                        /* File does not seem to be a Postscript one */
+                        
+                        /* We have to report error to the reader, do we do it here */
+                        g_print("File does not seem to be PostScript\n");
+                        if (gs->gs_filename)
+                                g_free(gs->gs_filename);
+                        if (gs->gs_filename_raw)
+                                g_free(gs->gs_filename_raw);
+                        if (gs->doc)
+                                psfree(gs->doc);
+                        gs->gs_filename_dsc = NULL;
+                        gs->gs_filename_unc = NULL;
+                        gs->doc = NULL;
+
+                        return FALSE;
+                        }
+
                 stat(gs->gs_filename, &sbuf);
                 gs->mtime = sbuf.st_mtime;
 	}
+        return TRUE;
 }
 
 
@@ -951,13 +1143,14 @@ gtk_gs_next_page(GtkGS *gs)
 	gs->busy = TRUE;
 	pw = (GdkWindowPrivate *)gs->widget.window;
 
+        g_print("Sending NEXT message to %ld\n", gs->message_window);
 	event.xclient.type          = ClientMessage;
 	event.xclient.display       = gdk_display;
-	event.xclient.window        = pw->xwindow;
+	event.xclient.window        = gs->message_window;
 	event.xclient.message_type  = gs_class->next_atom;
 	event.xclient.format        = 32;
         g_print ("sending event.");
-	gdk_send_xevent(pw->xwindow, FALSE, 0, &event);
+	gdk_send_xevent(gs->message_window, FALSE, 0, &event);
         g_print ("..sent\n");
         gdk_flush();
 	return TRUE;
@@ -999,21 +1192,24 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
         gs->current_page = page;
 
         
-        /* BIG BIG BIG FIXME!!!!! */
         /* 
          * For some reason, ghost script is unable to accept new input in
          * the next section.  So, we stop the interpreter and restart it.
          * I will look for this error later on!
+
+         * Update: This hack is history. This comment should be
+         removed as the fix is well tested.
+
          */
-        stop_interpreter (gs);
+        /*        stop_interpreter (gs); */
         /* End truly bad hack */
 
         if (gs->structured_doc) {
                 if (is_interpreter_ready (gs)) {
-                        /*g_print ("the interpreter is ready!\n");*/
+                       g_print ("the interpreter is ready!\n");
                         gtk_gs_next_page (gs);
                 } else {
-                        /*g_print ("the interpreter is not ready -- starting now\n");*/
+                        g_print ("the interpreter is not ready -- starting now\n");
                         gtk_gs_enable_interpreter (gs);
                         send_ps(gs, gs->doc->beginprolog, 
                                 gs->doc->lenprolog, FALSE);
@@ -1101,7 +1297,7 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gboolean set_default, gboole
                 gs->urx = new_urx;
                 gs->ury = new_ury;
                 changed = TRUE;
-                g_print ("setting up the page\n");
+                g_print ("setting up the page l: %d l:%d u:%d u:%d\n",new_llx,new_lly,new_urx,new_ury);
 
                 set_up_page(gs);
         }
@@ -1184,6 +1380,9 @@ compute_size(GtkGS *gs)
         change = (new_width != gs->width * gs->zoom_factor) || (new_height != gs->height * gs->zoom_factor);
         gs->width = (gint) (new_width * gs->zoom_factor);
         gs->height = (gint) (new_height * gs->zoom_factor);
+        g_print("GS: width %d height %d \n", gs->width, gs->height);
+	gtk_gs_munge_adjustments (gs);
+
         return (change);
 }
 
