@@ -774,6 +774,7 @@ gtk_scrollpane_motion_notify(GtkWidget * widget, GdkEventMotion * event)
 	  gdk_window_move(sp->slider, x, y);
 	}
 
+
       /* first we check the x adjustment... */
       value = ((sp->Xadjustment->upper - sp->Xadjustment->lower) *
 	       (x + ((gfloat) width) / 2) /
@@ -784,6 +785,10 @@ gtk_scrollpane_motion_notify(GtkWidget * widget, GdkEventMotion * event)
 	/*ugh -- yucky hack. to guarentee that value + page_size/2 < upper */
 	gtk_adjustment_set_value(sp->Xadjustment, sp->Xadjustment->upper - sp->Xadjustment->page_size / 2 - 0.0001);
 
+      /* g_print("SCRADJ: X: xu=%f xl=%f x=%d w=%d v=%f ps=%f value=%f\n",
+       *	sp->Xadjustment->upper, sp->Xadjustment->lower, x, width, value, sp->Xadjustment->page_size, value);
+       */
+
       /* then the y adjustment... */
       value = ((sp->Yadjustment->upper - sp->Yadjustment->lower) *
 	       (y + ((gfloat) height) / 2) /
