@@ -5,11 +5,14 @@
  */
 #include <config.h>
 #include <gnome.h>
+#include <math.h>
 #include <gdk/gdkkeysyms.h>
 #include "gtkscrollpane.h"
 #include "gtkgs.h"
 #include "crop.h"
 
+
+
 typedef struct _ggv_window ggv_window;
 struct _ggv_window {
         GtkWidget *main_window;
@@ -40,13 +43,14 @@ struct _window_with_data {
 
 GtkWidget *file_sel;
 GList *window_list;
+static int active_ggv;
 
 /* List of all files we are going to load at startup.  */
 static GList *garbage_can;
 
 /* prototypes */
-static void load_gs(ggv_window *ggv, char *fname);
-static ggv_window *create_ggv_window();
+static gboolean load_gs(ggv_window *ggv, char *fname);
+static ggv_window *create_ggv_window(void);
 static void create_menus(ggv_window *retval, window_with_data **temp);
 static void create_popup_menus(ggv_window *ggv);
 
@@ -54,6 +58,7 @@ static void create_popup_menus(ggv_window *ggv);
 static void about_callback(GtkWidget *widget, gpointer data);
 static void button_press_callback(GtkWidget *widget, GdkEventButton *event, gpointer data);
 static void button_release_callback(GtkWidget *widget, GdkEventButton *event, gpointer data);
+static void close_cmd (GtkWidget *widget, ggv_window *ggv);
 static gint exit_callback(GtkWidget *widget, gpointer data);
 static void file_open_destroy_callback(GtkWidget *widget, gpointer data);
 static void file_open_ok_callback(GtkWidget *widget, gpointer data);
@@ -76,18 +81,29 @@ static void save_callback(GtkWidget *widget, gpointer data);
 static void save_as_callback(GtkWidget *widget, gpointer data);
 static void select_page_callback(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data);
 static void zoom_callback(GtkWidget *widget, gpointer data);
+static double compute_zoom(gint zoom_spec);
 
 /* Usefull functions */
-static void
+static gboolean
 load_gs(ggv_window *ggv, char *fname)
 {
 	char *title;
 	guint i,n;
-
 	gchar *texts[3];
 
-        g_print ("loading the page...");
-	gtk_gs_load (GTK_GS (ggv->gs), fname);
+        g_print ("loading the page...zoom factor %f",compute_zoom(ggv->zoom_value));
+
+	if (!gtk_gs_load (GTK_GS (ggv->gs), fname)) {
+                GString *s;
+		GtkWidget *w;
+                s = g_string_new("Unable to load file ");
+                g_string_append(s, fname);
+                w =  gnome_app_error (GNOME_APP(ggv->main_window) , s->str);
+		gtk_widget_show(w);
+
+                g_string_free(s,0);
+                return FALSE;
+        }
         g_print ("loaded page...\n");
         fflush(NULL);
         gtk_gs_set_pagemedia(GTK_GS (ggv->gs), -1, FALSE, FALSE, 0);
@@ -103,6 +119,27 @@ load_gs(ggv_window *ggv, char *fname)
         else 
                 gtk_window_set_title (GTK_WINDOW(ggv->main_window), "ggv");
 
+        /* FIXME:
+           Resize window if necessary, we 
+           must verify whether automatic resizing is allowed (add option to
+           menu on whether to auto resize 
+           finally, at most it should be as large as the screen
+           
+           We also have to zoom the document to the specific zoom factor
+           according to the menu. 
+
+           gtk_gs_set_zoom (GTK_GS (ggv->gs), ggv->zoom_value); 
+
+           So far, the user can't make the window smaller. I don't 
+           know how to fix it.
+
+        */
+
+	/*
+        gtk_widget_set_usize (ggv->gs, 
+                              GTK_GS(ggv->gs)->width,GTK_GS(ggv->gs)->height);
+	*/
+
 	gtk_clist_freeze(GTK_CLIST(ggv->pagelist));
 	gtk_clist_clear(GTK_CLIST(ggv->pagelist));
         n = gtk_gs_document_numpages(GTK_GS (ggv->gs));
@@ -115,6 +152,7 @@ load_gs(ggv_window *ggv, char *fname)
 		}
 	}
 	gtk_clist_thaw(GTK_CLIST(ggv->pagelist));
+        return TRUE;
 }
 
 
@@ -124,6 +162,7 @@ load_gs(ggv_window *ggv, char *fname)
 static void about_callback(GtkWidget *widget, gpointer data) 
 {
 	GtkWidget *about = NULL;
+
 	gchar *authors[] = {
 		"Szekeres Istvan <szekeres@cyberspace.mht.bme.hu>",
                 "Jonathan Blandford <jrb@redhat.com>",
@@ -133,7 +172,7 @@ static void about_callback(GtkWidget *widget, gpointer data)
 	about = gnome_about_new(_("Ghostview for Gnome"), "0.1",
 				"Copyright (C) 1998 the Free Software Foundation",
 				(const gchar **) authors,
-				_("Postscript(TM) document viewer.\nBased on Tim Theisen's excellent Ghostview application."),
+				_("Postscript(TM) document viewer.\nBased on Tim Theisen's excellent Ghostview application (last modified by ."),
 				NULL);
         gtk_widget_show(about);
 }
@@ -146,7 +185,6 @@ static void button_press_callback(GtkWidget *widget, GdkEventButton *event, gpoi
 
 
         if (event->window == GTK_GS (ggv->gs)->pstarget) {
-                g_print ("in target...\n");
                 gdk_window_get_position (event->window, &x, &y);
         }
         x += event->x;
@@ -189,10 +227,43 @@ button_release_callback(GtkWidget *widget, GdkEventButton *event, gpointer data)
                 break;
         }
 }
+static void
+close_cmd (GtkWidget *widget, ggv_window *ggv)
+{
+        /* do we want to save changes??? */
+	window_list = g_list_remove (window_list, ggv);
+
+	gtk_widget_destroy (GTK_WIDGET (ggv->main_window));
+
+        active_ggv--;
+
+	if (active_ggv == 0)
+		gtk_main_quit ();
+
+}
+
+
 static gint
 exit_callback(GtkWidget *widget, gpointer data) 
 {
-        /* do we want to save changes??? */
+
+        ggv_window *ggv;
+
+        /* do we want to save anything before we exit??? */        
+
+        while (g_list_length(window_list)> 0) {
+                /* This is probably not the cleanest way
+                   to traverse the list, but I don't know any other yet,
+                   besides, it should not be a problem, the list should
+                   be very small anyways */
+
+                /*  g_print("List lenght %d\n", g_list_length(window_list)); */
+                ggv = (ggv_window*)g_list_nth_data (window_list,0);
+                window_list = g_list_remove (window_list, ggv);
+                gtk_widget_destroy (GTK_WIDGET (ggv->main_window));
+        }
+        
+
 	gtk_main_quit();
 
 	return 0;
@@ -210,10 +281,9 @@ static void file_open_ok_callback(GtkWidget *widget, gpointer data)
 
 	file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel));
 
-        /* FIXME: We need to confirm that it is open a la
-        if (*/	load_gs(ggv, file); /* ) {*/
-        ggv->loaded = TRUE;
-        /*}*/
+        if (load_gs(ggv, file))
+            ggv->loaded = TRUE;
+            
 }
 static void help_callback(GtkWidget *widget, gpointer data) {}
 static void
@@ -340,12 +410,12 @@ key_pressed_event_callback(GtkWidget *widget, GdkEventKey *event, gpointer data)
                 gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_Page_Up:
-                gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page+1);
+                gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page-1);
                 gtk_clist_select_row (GTK_CLIST (ggv->pagelist), GTK_GS (ggv->gs)->current_page,1);
                 gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_Page_Down:
-                gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page-1);
+                gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page+1);
                 gtk_clist_select_row (GTK_CLIST (ggv->pagelist), GTK_GS (ggv->gs)->current_page,1);
                 gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
@@ -355,7 +425,9 @@ static void next_page_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
 
+        g_print("Entering next page");
         if (ggv->loaded) {
+                g_print("going to next page");
                 gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page+1);
                 gtk_clist_select_row (GTK_CLIST (ggv->pagelist), GTK_GS (ggv->gs)->current_page,1);
         }
@@ -440,57 +512,59 @@ static void select_page_callback(GtkWidget *widget, gint row, gint column, GdkEv
 
         gtk_gs_goto_page (GTK_GS (ggv->gs), row);
 }
+
+static double compute_zoom(gint zoom_spec)
+{
+        double a,b;
+        if ((zoom_spec>>8) == 144) {
+                a = sqrt(2.0);
+        }
+        else {
+                a = (double)(zoom_spec>>8);
+        }
+        if ((zoom_spec & 0xff) == 144) {
+                b = sqrt(2.0);
+        }
+        else {
+                b = (double)(zoom_spec & 0xff);
+        }
+        return a/b;
+}
+
 static void zoom_callback(GtkWidget *widget, gpointer data)
 {
         window_with_data *wwd = (window_with_data *) data;
         ggv_window *ggv = wwd->ggv;
+        double z;
 
         if (ggv->zoom_value == wwd->data) 
                 return;
         ggv->zoom_value = wwd->data;
-        switch (wwd->data) {
-        case 1:
-                g_print ("case 1");
-                gtk_gs_set_zoom (GTK_GS (ggv->gs), 0.333);
-                gtk_gs_goto_page (GTK_GS (ggv->gs), GTK_GS(ggv->gs)->current_page);
-                break;
-        case 2:
-                g_print ("case 2");
-                gtk_gs_set_zoom (GTK_GS (ggv->gs), 0.5);
-                gtk_gs_goto_page (GTK_GS (ggv->gs), GTK_GS(ggv->gs)->current_page);
-                break;
-        case 3:
-                g_print ("case 3");
 
-                gtk_gs_set_zoom (GTK_GS (ggv->gs), 1.0);
-                gtk_gs_goto_page (GTK_GS (ggv->gs), GTK_GS(ggv->gs)->current_page);
-                break;
-        case 4:
-                g_print ("case 4");
+        z = compute_zoom(wwd->data);
 
-                gtk_gs_set_zoom (GTK_GS (ggv->gs), 2.0);
-                gtk_gs_goto_page (GTK_GS (ggv->gs), GTK_GS(ggv->gs)->current_page);
-                break;
-        case 5:
-                g_print ("case 5");
-                gtk_gs_set_zoom (GTK_GS (ggv->gs), 3.0);
-                gtk_gs_goto_page (GTK_GS (ggv->gs), GTK_GS(ggv->gs)->current_page);
-                break;
-        default:
+        g_print("Changing zoom mag: %f\n", z);
+
+        /* Return if no active doc */
+        if (!ggv->loaded) {
+                g_assert(ggv->gs);
+                GTK_GS(ggv->gs)->zoom_factor = z;
                 return;
         }
-
+        gtk_gs_set_zoom (GTK_GS (ggv->gs), z);
+        gtk_gs_goto_page (GTK_GS (ggv->gs), GTK_GS(ggv->gs)->current_page);        
 }
+
 static void
 create_popup_menus(ggv_window *ggv)
 {
         /* we do this the really hard way, unfortunately... */
 
-        GtkWidget *next = gtk_menu_item_new_with_label ("Next");
-        GtkWidget *prev = gtk_menu_item_new_with_label ("Prev");
+        GtkWidget *next = gtk_menu_item_new_with_label (_("Next"));
+        GtkWidget *prev = gtk_menu_item_new_with_label (_("Prev"));
         GtkWidget *separator = gtk_menu_item_new ();
-        ggv->show_hide_menubar = gtk_menu_item_new_with_label ("Hide Menubar");
-        ggv->show_hide_toolbar = gtk_menu_item_new_with_label ("Hide Side Panel");
+        ggv->show_hide_menubar = gtk_menu_item_new_with_label (_("Hide Menubar"));
+        ggv->show_hide_toolbar = gtk_menu_item_new_with_label (_("Hide Side Panel"));
         ggv->popup_menu = gtk_menu_new ();
 
         gtk_menu_append (GTK_MENU (ggv->popup_menu), next);
@@ -532,30 +606,28 @@ create_menus(ggv_window *retval, window_with_data **temp)
         GnomeUIInfo orientationMenu[] = {
                 {GNOME_APP_UI_RADIOITEMS, NULL , NULL, orientationMenu2, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL, NULL, NULL,
-                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL}
+                GNOMEUIINFO_END,
         };
         
         GnomeUIInfo zoomMenu2[] = {
-                {GNOME_APP_UI_ITEM, "1:3" , NULL, zoom_callback, temp[5], NULL,
-                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_ITEM, "1:2", NULL, zoom_callback, temp[6], NULL,
-                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_ITEM, "1:1", NULL, zoom_callback, temp[7], NULL,
-                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_ITEM, "2:1", NULL, zoom_callback, temp[8], NULL,
-                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_ITEM, "3:1", NULL, zoom_callback, temp[9], NULL,
-                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL, NULL, NULL,
-                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL}
+                GNOMEUIINFO_RADIOITEM_DATA("1:1", NULL, zoom_callback, temp[10], NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("1:10" , NULL, zoom_callback,temp[5] , NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("1:8" , NULL, zoom_callback,temp[6], NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("1:4", NULL, zoom_callback,temp[7], NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("1:2", NULL, zoom_callback,temp[8], NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("1:1.4", NULL, zoom_callback,temp[9], NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("1.4:1", NULL, zoom_callback,temp[11] , NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("2:1", NULL, zoom_callback,temp[12], NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("4:1", NULL, zoom_callback,temp[13], NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("8:1", NULL, zoom_callback,temp[14], NULL),
+                GNOMEUIINFO_RADIOITEM_DATA("10:1", NULL, zoom_callback,temp[15], NULL),
+                GNOMEUIINFO_END,
         };
         
         GnomeUIInfo zoomMenu[] = {
                 {GNOME_APP_UI_RADIOITEMS, NULL , NULL, zoomMenu2, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL, NULL, NULL,
-                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL}
+                GNOMEUIINFO_END,
         };
         
         GnomeUIInfo fileMenu[] = {
@@ -577,6 +649,9 @@ create_menus(ggv_window *retval, window_with_data **temp)
                 {GNOME_APP_UI_ITEM, N_("Recently Used Docs"), NULL, NULL, NULL, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN, 0, 0, NULL},
                 {GNOME_APP_UI_SEPARATOR},
+                {GNOME_APP_UI_ITEM, N_("Close"), NULL, close_cmd, retval, NULL,
+                 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 0, 0, NULL},
+                {GNOME_APP_UI_SEPARATOR},
                 {GNOME_APP_UI_ITEM, N_("Exit"), NULL, exit_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 0, 0, NULL},
                 {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL, NULL, NULL,
@@ -631,13 +706,16 @@ create_menus(ggv_window *retval, window_with_data **temp)
 	gnome_app_create_menus (GNOME_APP (retval->main_window), mainMenu);
 }        
 
+
+
 /* main functions */
+
 static
 ggv_window *create_ggv_window()
 {
         ggv_window *retval = g_malloc (sizeof (ggv_window));
         gint i;
-        window_with_data *temp[10];
+        window_with_data *temp[16];
         GtkWidget *pic;
         GtkWidget *npbhbox;
         GtkWidget *hbox;
@@ -647,11 +725,12 @@ ggv_window *create_ggv_window()
 
         /* create the main window. */
         retval->main_window = gnome_app_new("ggv", "ggv:");
+        retval->cd = NULL;
 	gtk_signal_connect (GTK_OBJECT (retval->main_window), "delete_event",
                             GTK_SIGNAL_FUNC (exit_callback), retval);
 
         /* Set up the callback values for the orientation */
-        for (i = 0; i < 10; i++) {
+        for (i = 0; i < 16; i++) {
                 temp[i] = g_malloc (sizeof (window_with_data));
                 temp[i]->ggv = retval;
                 garbage_can = g_list_append (garbage_can, temp[i]);
@@ -661,11 +740,23 @@ ggv_window *create_ggv_window()
         temp[2]->data = GTK_GS_ORIENTATION_LANDSCAPE;
         temp[3]->data = GTK_GS_ORIENTATION_SEASCAPE;
         temp[4]->data = GTK_GS_ORIENTATION_UPSIDEDOWN;
-        temp[5]->data = 1;
-        temp[6]->data = 2;
-        temp[7]->data = 3;
-        temp[8]->data = 4;
-        temp[9]->data = 5;
+
+        /* Two bytes a,b, they make up the ratio as in a/b.
+           sqrt(2) is special case, hence it is handled as
+           144 by zoom_callback */
+
+        temp[5]->data = 1 << 8 | 10;
+        temp[6]->data = 1 << 8 | 8;
+        temp[7]->data = 1 << 8 | 4;
+        temp[8]->data = 1 << 8 | 2;
+        temp[9]->data = 1 << 8 | 144 ;
+        temp[10]->data = 1 << 8 | 1;
+        temp[11]->data = 144 << 8 | 1;
+        temp[12]->data = 2 << 8 | 1;
+        temp[13]->data = 4 << 8 | 1;
+        temp[14]->data = 8 << 8 | 1;
+        temp[15]->data = 10 << 8 | 1 ;
+
         
         /* Menus: */
         create_menus (retval, temp);
@@ -683,8 +774,9 @@ ggv_window *create_ggv_window()
         /* we set up the post script display */
         retval->gs = gtk_gs_new (GTK_ADJUSTMENT (retval->hadj),
                                  GTK_ADJUSTMENT (retval->vadj));
-        gtk_widget_set_events (retval->gs, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK);
-        gtk_widget_set_usize (retval->gs, 300, 500);
+        gtk_widget_set_events (retval->gs, 
+                               GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK);
+        gtk_widget_set_usize (retval->gs, 400, 600);
         gtk_widget_show (retval->gs);
         
 
@@ -699,7 +791,7 @@ ggv_window *create_ggv_window()
         sbframe = gtk_frame_new (NULL);
         gtk_frame_set_shadow_type (GTK_FRAME (sbframe), GTK_SHADOW_NONE);
         sbvbox = gtk_vbox_new (FALSE, 0);
-        retval->zoom_value = 1;
+        retval->zoom_value = 1 << 8 | 1; /* 1 : 1 */
         retval->coordinates = gtk_label_new (" ");
         gtk_widget_show (retval->coordinates);
 
@@ -707,6 +799,7 @@ ggv_window *create_ggv_window()
         retval->scrollpane = gtk_scrollpane_new(GTK_ADJUSTMENT (retval->hadj), GTK_ADJUSTMENT (retval->vadj), 1.0);
         gtk_signal_connect (GTK_OBJECT(retval->scrollpane), "middle_clicked", GTK_SIGNAL_FUNC(scrollpane_middle_click_callback), retval);
         gtk_signal_connect (GTK_OBJECT(retval->scrollpane), "right_clicked", GTK_SIGNAL_FUNC(scrollpane_right_click_callback), retval);
+	gtk_widget_set_usize (retval->scrollpane, 80, 80);
         gtk_widget_show (retval->scrollpane);
         
         npbhbox = gtk_hbox_new (TRUE, 0);
@@ -763,6 +856,10 @@ ggv_window *create_ggv_window()
         return retval;
 }
 
+
+
+
+
 /* and we finally start the main prog. */
 int
 main(int argc, char *argv[])
@@ -771,6 +868,8 @@ main(int argc, char *argv[])
 	poptContext ctx;
 	char **startups;
 	int i;
+        
+        active_ggv=0;
 
         bindtextdomain (PACKAGE, GNOMELOCALEDIR);
         textdomain (PACKAGE);
@@ -783,15 +882,25 @@ main(int argc, char *argv[])
         if(startups) {
                 for(i = 0; startups[i]; i++) {
                         ggv_window *ggv = create_ggv_window ();
+                        g_print("Startup Options %s\n", startups[i]);
                         gtk_widget_show (ggv->main_window);
-                        load_gs (ggv, startups[i]);
+                        active_ggv ++;
+                        if (load_gs (ggv, startups[i])) {                                
+                                ggv->loaded = TRUE;
+                        }
+                        else {
+                                
+                        }
+                        /* Even if not loaded the window has been created */
                         window_list = g_list_append (window_list, ggv);
+
                 }
         }
 	poptFreeContext(ctx);
 
         if (window_list == NULL) {
                 ggv_window *ggv = create_ggv_window ();
+                active_ggv ++;
                 window_list = g_list_alloc();
                 window_list->data = ggv;
                 gtk_widget_show (ggv->main_window);
